@@ -8,7 +8,7 @@ from stretch_body.hello_utils import *
 class CollisionArmCamera(RobotCollisionModel):
     """
     NOTE: Experimental. You may want to turn this off in the params (enable=0)
-    RE1 camera can clip the arm when lift is all the way up
+    RE1/ RE2 camera can clip the arm when lift is all the way up
     and the camera is looking parallel to the ground.
     """
 
@@ -135,7 +135,7 @@ class CollisionStretchGripper(RobotCollisionModel):
     def dist_puller_past_base(self,x_arm,x_yaw):
         return self.dist_yaw_forward_of_base(x_arm) + self.dist_puller_forward_of_yaw(x_yaw)
 
-    def dist_palm_past_base(selfs,x_arm):
+    def dist_palm_past_base(self,x_arm):
         return x_arm-.04
 
 
