@@ -62,11 +62,14 @@ class TestDynamixelHelloXL430(unittest.TestCase):
 
         servo.stop()
 
+
     def test_valid_startup_status(self):
+        print('test_valid_startup_status')
         servo = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430(name="head_tilt", chain=None)
         self.assertTrue(servo.startup())
         self.assertNotEqual(servo.status['pos'],0)
 
+
     def test_non_multiturn_move_after_enable_pos(self):
         """Verify non-multiturn servo responds to move_to commands after enable_pos.
         """
@@ -90,6 +93,7 @@ class TestDynamixelHelloXL430(unittest.TestCase):
 
         servo.stop()
 
+
     def test_non_multiturn_move_after_home(self):
         """Verify non-multiturn servo responds to move_to commands after homing.
         """
@@ -106,6 +110,7 @@ class TestDynamixelHelloXL430(unittest.TestCase):
 
         servo.stop()
 
+
     def test_two_hardstop_homing(self):
         """Verify servo hits two hardstops during homing when single_stop=False.
         """
@@ -119,6 +124,7 @@ class TestDynamixelHelloXL430(unittest.TestCase):
 
         servo.stop()
 
+
     def test_runstop(self):
         print('test_runstop')
         class R():
@@ -146,6 +152,7 @@ class TestDynamixelHelloXL430(unittest.TestCase):
     def test_runstop_multithread(self):
         """Verify dynamixel_hello respect runstop via step_sentry
         """
+        print('test_runstop_multithread')
         servo = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430(name="head_pan", chain=None)
         self.assertTrue(servo.startup())
         servo.robot_params['robot_sentry']['dynamixel_stop_on_runstop'] = True
@@ -225,59 +232,61 @@ class TestDynamixelHelloXL430(unittest.TestCase):
         print('test_set_motion_profile')
         servo = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430(name="wrist_yaw", chain=None)
         self.assertTrue(servo.startup())
-
         # Check servo starts with profile equal to motion defaults
-        vel_ticks = servo.rad_per_sec_to_ticks(servo.params['motion']['default']['vel'])
-        accel_ticks = servo.rad_per_sec_sec_to_ticks(servo.params['motion']['default']['accel'])
-        self.assertEqual(servo.motor.get_profile_velocity(), vel_ticks)
-        self.assertEqual(servo.motor.get_profile_acceleration(), accel_ticks)
+
+        vel_ticks = servo.world_rad_to_ticks_per_sec(servo.params['motion']['default']['vel'])
+        self.assertEqual(servo.motor.get_profile_velocity(), abs(vel_ticks))
+        accel_ticks = servo.world_rad_to_ticks_per_sec_sec(servo.params['motion']['default']['accel'])
+        self.assertEqual(servo.motor.get_profile_acceleration(), abs(accel_ticks))
 
         # Set profile to zero
         vel_rad_sec = 0.0
         accel_rad_sec_sec = 0.0
-        vel_ticks = servo.rad_per_sec_to_ticks(vel_rad_sec)
-        accel_ticks = servo.rad_per_sec_sec_to_ticks(accel_rad_sec_sec)
+        vel_ticks = servo.world_rad_to_ticks_per_sec(vel_rad_sec)
+        accel_ticks = servo.world_rad_to_ticks_per_sec_sec(accel_rad_sec_sec)
         servo.set_motion_params(vel_rad_sec, accel_rad_sec_sec)
-        self.assertEqual(servo.motor.get_profile_velocity(), vel_ticks)
-        self.assertEqual(servo.motor.get_profile_acceleration(), accel_ticks)
+        self.assertEqual(servo.motor.get_profile_velocity(), abs(vel_ticks))
+        self.assertEqual(servo.motor.get_profile_acceleration(), abs(accel_ticks))
 
         # Set profile to slow
         vel_rad_sec = servo.params['motion']['slow']['vel']
         accel_rad_sec_sec = servo.params['motion']['slow']['accel']
-        vel_ticks = servo.rad_per_sec_to_ticks(vel_rad_sec)
-        accel_ticks = servo.rad_per_sec_sec_to_ticks(accel_rad_sec_sec)
+        vel_ticks = servo.world_rad_to_ticks_per_sec(vel_rad_sec)
+        accel_ticks = servo.world_rad_to_ticks_per_sec_sec(accel_rad_sec_sec)
         servo.set_motion_params(vel_rad_sec, accel_rad_sec_sec)
-        self.assertEqual(servo.motor.get_profile_velocity(), vel_ticks)
-        self.assertEqual(servo.motor.get_profile_acceleration(), accel_ticks)
+        self.assertEqual(servo.motor.get_profile_velocity(), abs(vel_ticks))
+        self.assertEqual(servo.motor.get_profile_acceleration(), abs(accel_ticks))
 
         # Set profile to fast
         vel_rad_sec = servo.params['motion']['fast']['vel']
         accel_rad_sec_sec = servo.params['motion']['fast']['accel']
-        vel_ticks = servo.rad_per_sec_to_ticks(vel_rad_sec)
-        accel_ticks = servo.rad_per_sec_sec_to_ticks(accel_rad_sec_sec)
+        vel_ticks = servo.world_rad_to_ticks_per_sec(vel_rad_sec)
+        accel_ticks = servo.world_rad_to_ticks_per_sec_sec(accel_rad_sec_sec)
         servo.set_motion_params(vel_rad_sec, accel_rad_sec_sec)
-        self.assertEqual(servo.motor.get_profile_velocity(), vel_ticks)
-        self.assertEqual(servo.motor.get_profile_acceleration(), accel_ticks)
+        self.assertEqual(servo.motor.get_profile_velocity(), abs(vel_ticks))
+        self.assertEqual(servo.motor.get_profile_acceleration(), abs(accel_ticks))
 
         # Set profile to max
         vel_rad_sec = servo.params['motion']['max']['vel']
         accel_rad_sec_sec = servo.params['motion']['max']['accel']
-        vel_ticks = servo.rad_per_sec_to_ticks(vel_rad_sec)
-        accel_ticks = servo.rad_per_sec_sec_to_ticks(accel_rad_sec_sec)
+        vel_ticks = servo.world_rad_to_ticks_per_sec(vel_rad_sec)
+        accel_ticks = servo.world_rad_to_ticks_per_sec_sec(accel_rad_sec_sec)
         servo.set_motion_params(vel_rad_sec, accel_rad_sec_sec)
-        self.assertEqual(servo.motor.get_profile_velocity(), vel_ticks)
-        self.assertEqual(servo.motor.get_profile_acceleration(), accel_ticks)
+        self.assertEqual(servo.motor.get_profile_velocity(), abs(vel_ticks))
+        self.assertEqual(servo.motor.get_profile_acceleration(), abs(accel_ticks))
 
         # Set profile to max plus more
         vel_rad_sec = servo.params['motion']['max']['vel'] + 1e5
         accel_rad_sec_sec = servo.params['motion']['max']['accel'] + 1e5
-        vel_ticks = servo.rad_per_sec_to_ticks(vel_rad_sec)
-        accel_ticks = servo.rad_per_sec_sec_to_ticks(accel_rad_sec_sec)
+        vel_ticks = servo.world_rad_to_ticks_per_sec(vel_rad_sec)
+        accel_ticks = servo.world_rad_to_ticks_per_sec_sec(accel_rad_sec_sec)
         servo.set_motion_params(vel_rad_sec, accel_rad_sec_sec)
-        self.assertNotEqual(servo.motor.get_profile_velocity(), vel_ticks)
-        self.assertNotEqual(servo.motor.get_profile_acceleration(), accel_ticks)
+        self.assertNotEqual(servo.motor.get_profile_velocity(), abs(vel_ticks))
+        self.assertNotEqual(servo.motor.get_profile_acceleration(), abs(accel_ticks))
 
         servo.stop()
+        
+
 
     def test_motion_profile_doesnt_persist(self):
         """Verify move_to/move_by with vel/accel doesn't set a motion profile that
@@ -289,8 +298,7 @@ class TestDynamixelHelloXL430(unittest.TestCase):
 
         servo.move_to(0.0)
         time.sleep(5)
-
-        servo.move_to(1.0, v_des=2.0, a_des=2.0)
+        servo.move_to(1.0, v_des=1.95, a_des=1.95)
         move1_vel_ticks = servo.motor.get_profile_velocity()
         move1_accel_ticks = servo.motor.get_profile_acceleration()
         time.sleep(5)
@@ -305,31 +313,35 @@ class TestDynamixelHelloXL430(unittest.TestCase):
 
         servo.stop()
 
+    
     def test_status_velocity(self):
         """Verify that the motor's velocity is correct in the status dict by
         commanding the robot a known distance and integrating velocity over
         time to confirm the two are approximately equal.
         """
+        print('test_status_velocity')
         s = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430(name="wrist_yaw")
         self.assertTrue(s.startup())
 
-        s.move_to(-1.1) # start at one end stop
+        s.move_to(-1.0) # start at one end stop
         time.sleep(5)
-        expected_traveled_distance = 4.3 + 1.1
+        expected_traveled_distance = 1.5 + 1.0
 
-        s.move_to(4.3, v_des=1.0, a_des=1.0) # travel to other end stop
-        ts = time.time()
+        s.move_to(1.5, v_des=1.0, a_des=1.0) # travel to other end stop
+        ts2 = time.time()
         measured_traveled_distance = 0.0
         s.pull_status()
-        while not np.isclose(s.status['pos'], 4.3, atol=1e-2):
+        ts=ts2
+        while not np.isclose(s.status['pos'], 1.5, atol=1e-2) and time.time()-ts2<6.0:
             dt = time.time() - ts
+            ts=time.time()
             measured_traveled_distance += s.status['vel'] * dt
-            ts = time.time()
             time.sleep(0.01)
             s.pull_status()
-            # print(s.status['pos'], expected_traveled_distance, measured_traveled_distance)
+            #print(dt,s.status['pos'], expected_traveled_distance, measured_traveled_distance)
 
         dt = time.time() - ts
         measured_traveled_distance += s.status['vel'] * dt
+        print('Measured',measured_traveled_distance)
         self.assertTrue(np.isclose(measured_traveled_distance, expected_traveled_distance, atol=0.3))
         s.stop()
